/* This program nices the command in the arguments and ensures that
   SIGXCPU is set to SIG_DFL.  I wish I knew how it gets set to SIG_IGN.

   If there are future problems with limits, the debugging stuff will
   be useful.
*/

#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <sys/resource.h>
#include <sys/types.h>
#include <sys/wait.h>

char *message[] =
{
  "",
  "Hangup",
  "Interrupt",
  "Quit",
  "Illegal instruction",
  "Trace trap",
  "Abort",
  "Bus error",
  "Floating-point exception",
  "Kill",
  "USR1",
  "Segmentation fault",
  "USR2",
  "Broken pipe",
  "Alarm",
  "Terminate",
  "Stack fault",
  "Child status has changed",
  "Continue",
  "Stop",
  "Keyboard stop",
  "Background read from tty",
  "Background write to tty",
  "Urgent condition on socket",
  "CPU limit exceeded",
  "File size limit exceeded",
  "Virtual alarm clock",
  "Profiling alarm clock",
  "Window size change",
  "I/O now possible",
  "Power failure restart",
  "Bad system call"
};

int main(int argc, char *argv[])
{
  //struct rlimit r;
  pid_t pid;
  const int runtime_ret = 1;
  const int timelimit_ret = 2;
  int num_signals = sizeof(message)/sizeof(char *);

  if ((pid = fork())) {
    int status, retval, sig;
    waitpid(pid, &status, 0);
    if (WIFEXITED(status)) {
      return 0;
    } else if (WIFSIGNALED(status)) {
      sig = WTERMSIG(status);
      if (sig == SIGKILL) {
	retval = timelimit_ret;
	sig = SIGXCPU;
      } else {
	retval = runtime_ret;
	printf("Run-time error: ");
      }
      if (sig < num_signals) {
        printf("%s\n", message[sig]);
      } else {
        printf("Signal %d\n", sig);
      }
      return retval;
    }
  } else if (!pid) {
    /* child */
    signal(SIGXCPU, SIG_DFL);
    //r.rlim_cur = r.rlim_max = atoi(argv[1]);
    //setrlimit(RLIMIT_CPU, &r);
    //r.rlim_cur = r.rlim_max = 0;
    //setrlimit(RLIMIT_CORE, &r);
    execv(argv[2], argv+2);
  } else {
    printf("cwrapper error: Cannot fork!\n");
  }
  return -1;
}
